/*!
 * mt3d-plugin-livecam v1.4.0
 * https://minitokyo3d.com
 * (c) 2021-2025 Akihiko Kusanagi
 * Released under the MIT license
 */
(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('mini-tokyo-3d')) :
typeof define === 'function' && define.amd ? define(['mini-tokyo-3d'], factory) :
(global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.mt3dLivecam = factory(global.mt3d));
})(this, (function (miniTokyo3d) { 'use strict';

var img = "data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 576 512'%3e%3c!--! Font Awesome Free 6.7.2 by %40fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free (Icons: CC BY 4.0%2c Fonts: SIL OFL 1.1%2c Code: MIT License) Copyright 2024 Fonticons%2c Inc. --%3e%3cpath d='M0 128C0 92.7 28.7 64 64 64l256 0c35.3 0 64 28.7 64 64l0 256c0 35.3-28.7 64-64 64L64 448c-35.3 0-64-28.7-64-64L0 128zM559.1 99.8c10.4 5.6 16.9 16.4 16.9 28.2l0 256c0 11.8-6.5 22.6-16.9 28.2s-23 5-32.9-1.6l-96-64L416 337.1l0-17.1 0-128 0-17.1 14.2-9.5 96-64c9.8-6.5 22.4-7.2 32.9-1.6z'/%3e%3c/svg%3e";

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".livecam-panel {\n    height: 262px !important;\n}\n.livecam-panel.collapsed {\n    height: 50px !important;\n}\n.livecam-panel.closed {\n    height: 0 !important;\n}\n.livecam-marker {\n    width: 40px;\n    height: 40px;\n    border: 2px solid #333;\n    border-radius: 50%;\n    background: white no-repeat center/20px url(\"data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 576 512' fill='%23333'%3E%3C!--! Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License) Copyright 2024 Fonticons, Inc. --%3E%3Cpath d='M0 128C0 92.7 28.7 64 64 64l256 0c35.3 0 64 28.7 64 64l0 256c0 35.3-28.7 64-64 64L64 448c-35.3 0-64-28.7-64-64L0 128zM559.1 99.8c10.4 5.6 16.9 16.4 16.9 28.2l0 256c0 11.8-6.5 22.6-16.9 28.2s-23 5-32.9-1.6l-96-64L416 337.1l0-17.1 0-128 0-17.1 14.2-9.5 96-64c9.8-6.5 22.4-7.2 32.9-1.6z'/%3E%3C/svg%3E\");\n    cursor: pointer;\n}\n.livecam-marker.active, .livecam-marker:hover {\n    border-color: #33B5E5;\n    background-image: url(\"data:image/svg+xml;charset=utf-8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 576 512' fill='%2333B5E5'%3E%3C!--! Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License) Copyright 2024 Fonticons, Inc. --%3E%3Cpath d='M0 128C0 92.7 28.7 64 64 64l256 0c35.3 0 64 28.7 64 64l0 256c0 35.3-28.7 64-64 64L64 448c-35.3 0-64-28.7-64-64L0 128zM559.1 99.8c10.4 5.6 16.9 16.4 16.9 28.2l0 256c0 11.8-6.5 22.6-16.9 28.2s-23 5-32.9-1.6l-96-64L416 337.1l0-17.1 0-128 0-17.1 14.2-9.5 96-64c9.8-6.5 22.4-7.2 32.9-1.6z'/%3E%3C/svg%3E\");\n}\n";
styleInject(css_248z);

// Live camera URL
const LIVECAM_URL = 'https://mini-tokyo.appspot.com/livecam';

// Refresh interval (5 minutes)
const REFRESH_INTERVAL = 300000;

function addColor(url, color) {
    const encodedColor = color.replace('#', '%23');
    return url.replace('%3e', ` fill=\'${encodedColor}\' stroke=\'${encodedColor}\'%3e`);
}

function createElement(tagName, attributes, container) {
    const element = document.createElement(tagName);

    Object.assign(element, attributes);
    return element;
}

class LivecamPanel extends miniTokyo3d.Panel {

    constructor(options) {
        super(Object.assign({className: 'livecam-panel'}, options));
    }

    addTo(map) {
        const me = this,
            {name, html} = me._options.camera;

        me.setTitle(name[map.lang] || name.en)
            .setHTML(html);
        return super.addTo(map);
    }

}

class LivecamPlugin {

    constructor() {
        const me = this;

        me.id = 'livecam';
        me.name = {
            de: 'Live-Kameras',
            en: 'Live Cameras',
            es: 'Cámaras en vivo',
            fr: 'Caméras en direct',
            ja: 'ライブカメラ',
            ko: '실시간 웹캠',
            ne: 'प्रत्यक्ष क्यामेरा',
            pt: 'Câmeras ao vivo',
            th: 'กล้องถ่ายทอดสด',
            'zh-Hans': '实时摄像头',
            'zh-Hant': '實時攝像頭'
        };
        me.iconStyle = {
            backgroundSize: '32px',
            backgroundImage: `url("${addColor(img, 'white')}")`
        };
        me.clockModes = ['realtime'];
        me.viewModes = ['ground'];
        me.cameras = {};
        me._onSelection = me._onSelection.bind(me);
        me._onDeselection = me._onDeselection.bind(me);
    }

    onAdd(map) {
        this.map = map;
    }

    onEnabled() {
        const me = this,
            map = me.map;

        map.on('selection', me._onSelection);
        map.on('deselection', me._onDeselection);

        const repeat = () => {
            const now = performance.now();

            if (Math.floor(now / REFRESH_INTERVAL) !== Math.floor(me._lastCameraRefresh / REFRESH_INTERVAL)) {
                fetch(LIVECAM_URL)
                    .then(response => response.json())
                    .then(data => me._updateCameras(data));
                me._lastCameraRefresh = now;
            }
            me._frameRequestID = requestAnimationFrame(repeat);
        };

        repeat();
    }

    onDisabled() {
        const me = this,
            {map, panel} = me;

        map.off('selection', me._onSelection);
        map.off('deselection', me._onDeselection);

        if (panel) {
            map.trackObject();
            panel.remove();
            delete me.panel;
        }

        cancelAnimationFrame(me._frameRequestID);
        delete me._lastCameraRefresh;

        me._updateCameras([]);
    }

    onVisibilityChanged(visible) {
        const me = this,
            {map, panel, cameras} = me;

        me.visible = visible;

        if (!visible && panel) {
            map.trackObject();
        }
        for (const id of Object.keys(cameras)) {
            cameras[id].marker.setVisibility(visible);
        }
    }

    _updateCameras(data) {
        const me = this,
            {map, cameras, visible} = me,
            lang = map.lang;

        for (const item of data) {
            const id = item.id,
                camera = cameras[id];

            if (camera) {
                Object.assign(camera, item);
                camera.marker.setLngLat(item.center);
                camera.updated = true;
                continue;
            }

            const element = createElement('div', {className: 'livecam-marker'}),
                marker = new miniTokyo3d.Marker({element, minZoom: 11})
                    .setLngLat(item.center)
                    .addTo(map)
                    .setVisibility(visible)
                    .on('click', () => {
                        const {center, zoom, bearing, pitch} = cameras[id];

                        map.trackObject({id, selectionType: 'livecam'});
                        map.getMapboxMap().flyTo({center, zoom, bearing, pitch});
                    })
                    .on('mouseenter', () => {
                        const {center, name, thumbnail} = cameras[id];

                        cameras[id].popup = new miniTokyo3d.Popup()
                            .setLngLat(center)
                            .setHTML([
                                '<div class="thumbnail-image-container">',
                                '<div class="ball-pulse"><div></div><div></div><div></div></div>',
                                `<div class="thumbnail-image" style="background-image: url(\'${thumbnail}\');"></div>`,
                                '</div>',
                                `<div><strong>${name[lang] || name.en}</strong></div>`
                            ].join(''))
                            .addTo(map);
                    })
                    .on('mouseleave', () => {
                        if (cameras[id].popup) {
                            cameras[id].popup.remove();
                            delete cameras[id].popup;
                        }
                    });

            cameras[id] = Object.assign({marker, updated: true}, item);
        }

        for (const id of Object.keys(cameras)) {
            if (cameras[id].updated) {
                delete cameras[id].updated;
            } else {
                if (cameras[id].popup) {
                    cameras[id].popup.remove();
                }
                cameras[id].marker.remove();
                delete cameras[id];
            }
        }
    }

    _onSelection(event) {
        if (event.selectionType === 'livecam') {
            const me = this,
                camera = me.cameras[event.id];

            camera.marker.setActivity(true);
            me.panel = new LivecamPanel({camera}).addTo(me.map);
        }
    }

    _onDeselection(event) {
        if (event.selectionType === 'livecam') {
            const me = this,
                camera = me.cameras[event.id],
                panel = me.panel;

            if (camera && camera.marker) {
                camera.marker.setActivity(false);
            }
            if (panel) {
                panel.remove();
                delete me.panel;
            }
        }
    }

}

function index() {
    return new LivecamPlugin();
}

return index;

}));
//# sourceMappingURL=mt3d-plugin-livecam.js.map
